#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re, sys, os, getopt
from checkutils import *

def read_group_from_line(l):
    if l[0]=='[':
        gname=l[1:].split(']')[0]
    else: gname=''
    return gname

def print_groups_from_file(patternfile):
    lines=codecs.open(patternfile,"rb","utf8").readlines()
    print 'groups in %s:'%patternfile
    for l in lines:
        gname = read_group_from_line(l)
        if gname!='':
            print gname

def read_pattern_from_file(patternfile,group='all'):
    current_group,suchmuster, comment = '',[], []
    lines=codecs.open(patternfile,"rb","utf8").readlines()
    for l in lines:
        if l.strip()!='': # leere zeilen raus
            if l.strip()[0]!='#': # kommentar '#' raus
                gname = read_group_from_line(l)
                if gname!='':
                    current_group=gname
                elif current_group==group or group=='all':
                    if len(l.split('"'))==3:
                        suchmuster.append(l.split('"')[1])  # "regexpression"
                        comment.append(l.split('"')[2].strip())  # "comment for pattern"
                    else:
                        print 'syntax error %s in patternfile' %patternfile
    return suchmuster, comment

def usage():
    print '\nUsage: python %s [OPTION] path/to/[app[doc.po]]' %os.path.basename(sys.argv[0])
    print '       Find pattern in msgstr, print or edit them '
    print 'Options: -h, --help  : usage'
    print '         -k, --konsole : only output to konsole, no edit of *.po files'
    print '         -p file, --pattern file : read pattern from file (default patternkzwiebelfisch)'
    print '         -r "regex1|regex2|etc", --regex "regex1|regex2|etc": use regex1,regex2,etc (quoted+separated with "|") as pattern'
    print '         -g groupname, --group groupname : read only pattern in groupname from file (default patternkzwiebelfisch)'
    print '         -g ?, --group ? : print all groups from file (default patternkzwiebelfisch)'
    print 'Default : read pattern in all groups from file patternkzwiebelfisch, edit msgstr with python-kde3-dialog\n'
    sys.exit(2)
try:
    opts, args = getopt.getopt(sys.argv[1:], "hkp:r:g:", ["help", "konsole", 'pattern=', 'regex=', 'group='])
except getopt.GetoptError:
    usage() # print help information and exit

konsole = False     # only output to konsole
dialog = True       # edit with dialog
suchmuster,comment =[], []
group='all'
print_groups = False
patternfile='patternkzwiebelfisch'

for o,a in opts:
    if o in ("-h", "--help"):
        usage()
    if o in ("-k", "--konsole"):
        konsole=True
        dialog = False
    if o in ("-g", "--group"):
        if a=='?':
            print_groups=True 
        else:
            group=a
    if o in ("-p", "--pattern"):
            patternfile=a
    if o in ("-r", "--regex"):
        print_groups=False
        suchmuster=unicode(a,sys.stdout.encoding or sys.getfilesystemencoding()).split('|')
        for i in range(len(suchmuster)):
            comment.append('no comment')

if len(args) != 1 and print_groups==False:
    usage()

if not os.path.isfile(patternfile): #absolute path to patternfile?
    patternfile='%s/%s' %(os.path.dirname(sys.argv[0]),patternfile) #try path of kzwiebelfisch.py
if not os.path.isfile(patternfile):
    sys.exit('patternfile %s not found' %a) 
if print_groups:
    try:
        print_groups_from_file(patternfile)
    except:
        sys.exit('%s is no patternfile' %patternfile)
    sys.exit()

if suchmuster==[]:
    try:
        suchmuster,comment=read_pattern_from_file(patternfile,group)
    except:
        sys.exit('error in patternfile %s' %patternfile)

pofilelist=getfilelistfromdir(args[0],'.po')
if pofilelist==[]:
    print '\n%s is no path to dir with *.po files or no *.po file' %args[0]
    usage()
pofilenrpatternlist=[]
sumtrefferzahl=0
for pofile in pofilelist:
    trefferzahl=0
    if os.access(pofile, os.W_OK):
        po=guidocpo(pofile)
        for i in po.messagerange():
            fundmuster,patternlist='',[]
            for j in range(len(suchmuster)): #search all pattern in msgstr
                such=re.compile(suchmuster[j],re.DOTALL)
                if po.is_plural(i):
                    msg=' '.join(po.msgstr[i])
                else: msg=po.msgstr[i]
                if such.findall(msg):
                    trefferzahl+=1
                    if suchmuster[j] not in fundmuster: 
                        fundmuster+='>%s< %s\n' %(suchmuster[j],comment[j])
                        patternlist.append([suchmuster[j],comment[j]])
            if fundmuster!='':
                if dialog:
                    pofilenrpatternlist.append([po, i, patternlist])
                else:
                    print '\n%s #%d pattern: %s' %(po.name,i,fundmuster)
                    print 'msgid: %s\nmsgstr: %s' %(po.msgid[i],po.msgstr[i])
        if trefferzahl>0:
            sumtrefferzahl+=trefferzahl
            print 'Result: %i possible errors found in %s' %(trefferzahl,po.name)
        #else: print 'Result: no errors found in %s' %po.name
    else:
        print 'No write access to %s, file skipped' %pofile
if sumtrefferzahl>0:
    if len(pofilelist)>1:
        print '\nSummary: %i possible errors found in %i files\n'\
               %(sumtrefferzahl,len(pofilelist))
    if dialog:
        from kdecore import KApplication
        from qt import SIGNAL,SLOT,QDialog
        sys.argv=sys.argv[0:1]
        app = KApplication (sys.argv,"checkdialog")
        mainWindow = MainDialog (None,pofilenrpatternlist,title = "kzwiebelfisch")
        app.setMainWidget(mainWindow)
        mainWindow.resize(768, 640)
        mainWindow.show()
        #app.connect( app, SIGNAL('lastWindowClosed()'), app, SLOT('quit()') )
        app.exec_loop()
        if mainWindow.result() == QDialog.Accepted:
            for i in pofilenrpatternlist:
                i[0].saveclose()
            print "saved all changed message catalogues"
        else:
            print "discard all changes"
